package cn.ibizlab.businesscentral.core.odoo_hr.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.aop.framework.AopContext;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_employee_skill;
import cn.ibizlab.businesscentral.core.odoo_hr.filter.Hr_employee_skillSearchContext;
import cn.ibizlab.businesscentral.core.odoo_hr.service.IHr_employee_skillService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.util.helper.EBSServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_hr.mapper.Hr_employee_skillMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;

import org.springframework.util.StringUtils;

/**
 * 实体[员工技能] 服务对象接口实现
 */
@Slf4j
@Service("Hr_employee_skillServiceImpl")
public class Hr_employee_skillServiceImpl extends EBSServiceImpl<Hr_employee_skillMapper, Hr_employee_skill> implements IHr_employee_skillService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_hr.service.IHr_employeeService hrEmployeeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_hr.service.IHr_skill_levelService hrSkillLevelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_hr.service.IHr_skillService hrSkillService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_hr.service.IHr_skill_typeService hrSkillTypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    public String getIrModel(){
        return "hr.employee.skill" ;
    }

    private boolean messageinfo = false ;

    public void setMessageInfo(boolean messageinfo){
        this.messageinfo = messageinfo ;
    }

    @Override
    @Transactional
    public boolean create(Hr_employee_skill et) {
        boolean mail_create_nosubscribe = et.get("mail_create_nosubscribe") != null;
        boolean mail_create_nolog = et.get("mail_create_nolog") != null;
        boolean mail_notrack = et.get("mail_notrack") != null;
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (IHr_employee_skillService)AopContext.currentProxy() : this).get(et.getId()),et);

        if (messageinfo && !mail_create_nosubscribe) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_followersExService.class).add_default_followers(this,et);
        }

        if (messageinfo && !mail_create_nolog) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_messageExService.class).add_default_create_message(this,et);
        }

        if (messageinfo && !mail_notrack) {

        }
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Hr_employee_skill> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Hr_employee_skill et) {
        Hr_employee_skill old = new Hr_employee_skill() ;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (IHr_employee_skillService)AopContext.currentProxy() : this).get(et.getId()), old);
        boolean mail_notrack = et.get("mail_notrack") != null;
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (IHr_employee_skillService)AopContext.currentProxy() : this).get(et.getId()),et);
        if (messageinfo && !mail_notrack) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_tracking_valueExService.class).message_track(this,old,et);
        }
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Hr_employee_skill> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Hr_employee_skill get(Long key) {
        Hr_employee_skill et = getById(key);
        if(et==null){
            et=new Hr_employee_skill();
            et.setId(key);
        }
        else{
        }
        return et;
    }

    @Override
    public Hr_employee_skill getDraft(Hr_employee_skill et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(Hr_employee_skill et) {
        return (!ObjectUtils.isEmpty(et.getId()))&&(!Objects.isNull(this.getById(et.getId())));
    }
    @Override
    @Transactional
    public boolean save(Hr_employee_skill et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(Hr_employee_skill et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<Hr_employee_skill> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<Hr_employee_skill> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<Hr_employee_skill> selectByEmployeeId(Long id) {
        return baseMapper.selectByEmployeeId(id);
    }
    @Override
    public void removeByEmployeeId(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("employee_id",id));
    }

	@Override
    public List<Hr_employee_skill> selectBySkillLevelId(Long id) {
        return baseMapper.selectBySkillLevelId(id);
    }
    @Override
    public void removeBySkillLevelId(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("skill_level_id",id));
    }

	@Override
    public List<Hr_employee_skill> selectBySkillId(Long id) {
        return baseMapper.selectBySkillId(id);
    }
    @Override
    public void removeBySkillId(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("skill_id",id));
    }

	@Override
    public List<Hr_employee_skill> selectBySkillTypeId(Long id) {
        return baseMapper.selectBySkillTypeId(id);
    }
    @Override
    public void removeBySkillTypeId(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("skill_type_id",id));
    }

	@Override
    public List<Hr_employee_skill> selectByCreateUid(Long id) {
        return baseMapper.selectByCreateUid(id);
    }
    @Override
    public void removeByCreateUid(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("create_uid",id));
    }

	@Override
    public List<Hr_employee_skill> selectByWriteUid(Long id) {
        return baseMapper.selectByWriteUid(id);
    }
    @Override
    public void removeByWriteUid(Long id) {
        this.remove(new QueryWrapper<Hr_employee_skill>().eq("write_uid",id));
    }


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<Hr_employee_skill> searchDefault(Hr_employee_skillSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Hr_employee_skill> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Hr_employee_skill>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Hr_employee_skill et){
        //实体关系[DER1N_HR_EMPLOYEE_SKILL_HR_EMPLOYEE_EMPLOYEE_ID]
        if(!ObjectUtils.isEmpty(et.getEmployeeId())){
            cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_employee odooEmployee=et.getOdooEmployee();
            if(ObjectUtils.isEmpty(odooEmployee)){
                cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_employee majorEntity=hrEmployeeService.get(et.getEmployeeId());
                et.setOdooEmployee(majorEntity);
                odooEmployee=majorEntity;
            }
            et.setEmployeeName(odooEmployee.getName());
        }
        //实体关系[DER1N_HR_EMPLOYEE_SKILL_HR_SKILL_LEVEL_SKILL_LEVEL_ID]
        if(!ObjectUtils.isEmpty(et.getSkillLevelId())){
            cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill_level odooSkillLevel=et.getOdooSkillLevel();
            if(ObjectUtils.isEmpty(odooSkillLevel)){
                cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill_level majorEntity=hrSkillLevelService.get(et.getSkillLevelId());
                et.setOdooSkillLevel(majorEntity);
                odooSkillLevel=majorEntity;
            }
            et.setSkillLevelName(odooSkillLevel.getName());
        }
        //实体关系[DER1N_HR_EMPLOYEE_SKILL_HR_SKILL_SKILL_ID]
        if(!ObjectUtils.isEmpty(et.getSkillId())){
            cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill odooSkill=et.getOdooSkill();
            if(ObjectUtils.isEmpty(odooSkill)){
                cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill majorEntity=hrSkillService.get(et.getSkillId());
                et.setOdooSkill(majorEntity);
                odooSkill=majorEntity;
            }
            et.setSkillName(odooSkill.getName());
        }
        //实体关系[DER1N_HR_EMPLOYEE_SKILL_HR_SKILL_TYPE_SKILL_TYPE_ID]
        if(!ObjectUtils.isEmpty(et.getSkillTypeId())){
            cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill_type odooSkillType=et.getOdooSkillType();
            if(ObjectUtils.isEmpty(odooSkillType)){
                cn.ibizlab.businesscentral.core.odoo_hr.domain.Hr_skill_type majorEntity=hrSkillTypeService.get(et.getSkillTypeId());
                et.setOdooSkillType(majorEntity);
                odooSkillType=majorEntity;
            }
            et.setSkillTypeName(odooSkillType.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }




}



