package cn.ibizlab.businesscentral.core.odoo_lunch.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.aop.framework.AopContext;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_order_line;
import cn.ibizlab.businesscentral.core.odoo_lunch.filter.Lunch_order_lineSearchContext;
import cn.ibizlab.businesscentral.core.odoo_lunch.service.ILunch_order_lineService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.util.helper.EBSServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_lunch.mapper.Lunch_order_lineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;

import org.springframework.util.StringUtils;

/**
 * 实体[工作餐订单行] 服务对象接口实现
 */
@Slf4j
@Service("Lunch_order_lineServiceImpl")
public class Lunch_order_lineServiceImpl extends EBSServiceImpl<Lunch_order_lineMapper, Lunch_order_line> implements ILunch_order_lineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_lunch.service.ILunch_cashmoveService lunchCashmoveService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_lunch.service.ILunch_orderService lunchOrderService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_lunch.service.ILunch_product_categoryService lunchProductCategoryService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_lunch.service.ILunch_productService lunchProductService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_partnerService resPartnerService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    public String getIrModel(){
        return "lunch.order.line" ;
    }

    private boolean messageinfo = false ;

    public void setMessageInfo(boolean messageinfo){
        this.messageinfo = messageinfo ;
    }

    @Override
    @Transactional
    public boolean create(Lunch_order_line et) {
        boolean mail_create_nosubscribe = et.get("mail_create_nosubscribe") != null;
        boolean mail_create_nolog = et.get("mail_create_nolog") != null;
        boolean mail_notrack = et.get("mail_notrack") != null;
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (ILunch_order_lineService)AopContext.currentProxy() : this).get(et.getId()),et);

        if (messageinfo && !mail_create_nosubscribe) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_followersExService.class).add_default_followers(this,et);
        }

        if (messageinfo && !mail_create_nolog) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_messageExService.class).add_default_create_message(this,et);
        }

        if (messageinfo && !mail_notrack) {

        }
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Lunch_order_line> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Lunch_order_line et) {
        Lunch_order_line old = new Lunch_order_line() ;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (ILunch_order_lineService)AopContext.currentProxy() : this).get(et.getId()), old);
        boolean mail_notrack = et.get("mail_notrack") != null;
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy((AopContext.currentProxy() != null ? (ILunch_order_lineService)AopContext.currentProxy() : this).get(et.getId()),et);
        if (messageinfo && !mail_notrack) {
            cn.ibizlab.businesscentral.util.security.SpringContextHolder.getBean(cn.ibizlab.businesscentral.core.extensions.service.Mail_tracking_valueExService.class).message_track(this,old,et);
        }
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Lunch_order_line> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        lunchCashmoveService.removeByOrderId(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        lunchCashmoveService.removeByOrderId(idList);
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Lunch_order_line get(Long key) {
        Lunch_order_line et = getById(key);
        if(et==null){
            et=new Lunch_order_line();
            et.setId(key);
        }
        else{
        }
        return et;
    }

    @Override
    public Lunch_order_line getDraft(Lunch_order_line et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(Lunch_order_line et) {
        return (!ObjectUtils.isEmpty(et.getId()))&&(!Objects.isNull(this.getById(et.getId())));
    }
    @Override
    @Transactional
    public boolean save(Lunch_order_line et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(Lunch_order_line et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<Lunch_order_line> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<Lunch_order_line> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<Lunch_order_line> selectByOrderId(Long id) {
        return baseMapper.selectByOrderId(id);
    }
    @Override
    public void removeByOrderId(Collection<Long> ids) {
        this.remove(new QueryWrapper<Lunch_order_line>().in("order_id",ids));
    }

    @Override
    public void removeByOrderId(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("order_id",id));
    }

	@Override
    public List<Lunch_order_line> selectByCategoryId(Long id) {
        return baseMapper.selectByCategoryId(id);
    }
    @Override
    public void resetByCategoryId(Long id) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("category_id",null).eq("category_id",id));
    }

    @Override
    public void resetByCategoryId(Collection<Long> ids) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("category_id",null).in("category_id",ids));
    }

    @Override
    public void removeByCategoryId(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("category_id",id));
    }

	@Override
    public List<Lunch_order_line> selectByProductId(Long id) {
        return baseMapper.selectByProductId(id);
    }
    @Override
    public void resetByProductId(Long id) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("product_id",null).eq("product_id",id));
    }

    @Override
    public void resetByProductId(Collection<Long> ids) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("product_id",null).in("product_id",ids));
    }

    @Override
    public void removeByProductId(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("product_id",id));
    }

	@Override
    public List<Lunch_order_line> selectBySupplier(Long id) {
        return baseMapper.selectBySupplier(id);
    }
    @Override
    public void resetBySupplier(Long id) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("supplier",null).eq("supplier",id));
    }

    @Override
    public void resetBySupplier(Collection<Long> ids) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("supplier",null).in("supplier",ids));
    }

    @Override
    public void removeBySupplier(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("supplier",id));
    }

	@Override
    public List<Lunch_order_line> selectByCreateUid(Long id) {
        return baseMapper.selectByCreateUid(id);
    }
    @Override
    public void removeByCreateUid(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("create_uid",id));
    }

	@Override
    public List<Lunch_order_line> selectByUserId(Long id) {
        return baseMapper.selectByUserId(id);
    }
    @Override
    public void resetByUserId(Long id) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("user_id",null).eq("user_id",id));
    }

    @Override
    public void resetByUserId(Collection<Long> ids) {
        this.update(new UpdateWrapper<Lunch_order_line>().set("user_id",null).in("user_id",ids));
    }

    @Override
    public void removeByUserId(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("user_id",id));
    }

	@Override
    public List<Lunch_order_line> selectByWriteUid(Long id) {
        return baseMapper.selectByWriteUid(id);
    }
    @Override
    public void removeByWriteUid(Long id) {
        this.remove(new QueryWrapper<Lunch_order_line>().eq("write_uid",id));
    }


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<Lunch_order_line> searchDefault(Lunch_order_lineSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Lunch_order_line> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Lunch_order_line>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Lunch_order_line et){
        //实体关系[DER1N_LUNCH_ORDER_LINE__LUNCH_ORDER__ORDER_ID]
        if(!ObjectUtils.isEmpty(et.getOrderId())){
            cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_order odooOrder=et.getOdooOrder();
            if(ObjectUtils.isEmpty(odooOrder)){
                cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_order majorEntity=lunchOrderService.get(et.getOrderId());
                et.setOdooOrder(majorEntity);
                odooOrder=majorEntity;
            }
            et.setCurrencyId(odooOrder.getCurrencyId());
            et.setDate(odooOrder.getDate());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__LUNCH_PRODUCT_CATEGORY__CATEGORY_ID]
        if(!ObjectUtils.isEmpty(et.getCategoryId())){
            cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_product_category odooCategory=et.getOdooCategory();
            if(ObjectUtils.isEmpty(odooCategory)){
                cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_product_category majorEntity=lunchProductCategoryService.get(et.getCategoryId());
                et.setOdooCategory(majorEntity);
                odooCategory=majorEntity;
            }
            et.setCategoryIdText(odooCategory.getName());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__LUNCH_PRODUCT__PRODUCT_ID]
        if(!ObjectUtils.isEmpty(et.getProductId())){
            cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_product odooProduct=et.getOdooProduct();
            if(ObjectUtils.isEmpty(odooProduct)){
                cn.ibizlab.businesscentral.core.odoo_lunch.domain.Lunch_product majorEntity=lunchProductService.get(et.getProductId());
                et.setOdooProduct(majorEntity);
                odooProduct=majorEntity;
            }
            et.setPrice(odooProduct.getPrice());
            et.setName(odooProduct.getName());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__RES_PARTNER__SUPPLIER]
        if(!ObjectUtils.isEmpty(et.getSupplier())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner odooSupplier=et.getOdooSupplier();
            if(ObjectUtils.isEmpty(odooSupplier)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner majorEntity=resPartnerService.get(et.getSupplier());
                et.setOdooSupplier(majorEntity);
                odooSupplier=majorEntity;
            }
            et.setSupplierText(odooSupplier.getName());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__RES_USERS__USER_ID]
        if(!ObjectUtils.isEmpty(et.getUserId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooUser=et.getOdooUser();
            if(ObjectUtils.isEmpty(odooUser)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getUserId());
                et.setOdooUser(majorEntity);
                odooUser=majorEntity;
            }
            et.setUserIdText(odooUser.getName());
        }
        //实体关系[DER1N_LUNCH_ORDER_LINE__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Lunch_order_line> getLunchOrderLineByIds(List<Long> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Lunch_order_line> getLunchOrderLineByEntities(List<Lunch_order_line> entities) {
        List ids =new ArrayList();
        for(Lunch_order_line entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



